from made.chat_env.repository.chat_env_repository_impl import ChatEnvRepositoryImpl
from made.engine import ModelConfig
from made.phase import PhaseRegistry
from made.phase.entity.phase_chat_turn_limit import PhaseChatTurnLimit
from made.phase.entity.phase_engine_config import PhaseEngineConfig
from made.phase.entity.phase_prompts import PhasePrompt
from made.phase.repository.base_phase_repository_impl import BasePhaseRepositoryImpl
from made.role_playing.entity.role_prompts import RolePrompt
from made.role_playing.entity.role_type import RoleType


@PhaseRegistry.register()
class LanguageChoosePhaseRepositoryImpl(BasePhaseRepositoryImpl):
    def __init__(
        self,
        model_config: ModelConfig,
        phase_prompt: str = PhasePrompt.language_choose,
        assistant_role_name: str = RoleType.CTO,
        assistant_role_prompt: str = RolePrompt.CTO,
        user_role_name: str = RoleType.CEO,
        user_role_prompt: str = RolePrompt.CEO,
        chat_turn_limit: int = PhaseChatTurnLimit.language_choose,
        **kwargs
    ):
        phase_engine_config = PhaseEngineConfig.language_choose
        super().__init__(
            model_config=model_config,
            phase_prompt=phase_prompt,
            assistant_role_name=assistant_role_name,
            assistant_role_prompt=assistant_role_prompt,
            user_role_name=user_role_name,
            user_role_prompt=user_role_prompt,
            chat_turn_limit=chat_turn_limit,
            temperature=phase_engine_config.temperature,
            top_p=phase_engine_config.top_p,
            **kwargs
        )

    def update_phase_states(self, env: ChatEnvRepositoryImpl):
        # TODO implement
        self.states.task = env.config.task_prompt
        self.states.modality = env.states.modality

    def update_env_states(self, env: ChatEnvRepositoryImpl) -> ChatEnvRepositoryImpl:
        env.states.language = self.seminar_conclusion
        return env



from typing import Union

from made.messages.entity.base_message import BaseMessage
from made.messages.entity.chat_message.assistant_message import AssistantChatMessage
from made.messages.entity.chat_message.base_chat_message import BaseChatMessage
from made.messages.entity.chat_message.user_message import UserChatMessage
from made.messages.entity.system_message.assistant_system_message import (
    AssistantSystemMessage,
)
from made.messages.entity.system_message.base_system_message import BaseSystemMessage
from made.messages.entity.system_message.user_system_message import UserSystemMessage


MessageType = Union[
    BaseMessage,
    AssistantChatMessage,
    BaseChatMessage,
    UserChatMessage,
    AssistantSystemMessage,
    BaseSystemMessage,
    UserSystemMessage,
]

SystemMessageType = Union[AssistantSystemMessage, BaseSystemMessage, UserSystemMessage]

ChatMessageType = Union[AssistantChatMessage, BaseChatMessage, UserChatMessage]



class Registry:
    def __init__(self, name):
        self._name = name
        self._obj_map = {}

    def _do_register(self, name, obj):
        assert name not in self._obj_map, (
            f"An object named '{name}' was already registered "
            f"in '{self._name}' registry!"
        )
        if "RepositoryImpl" in name:
            name = name.replace("RepositoryImpl", "")
        self._obj_map[name] = obj

    def register(self, obj=None):
        if obj is None:

            def deco(func_or_class):
                name = func_or_class.__name__
                self._do_register(name, func_or_class)
                return func_or_class

            return deco
        name = obj.__name__
        self._do_register(name, obj)

    def get(self, name):
        if name in self._obj_map.keys():
            ret = self._obj_map.get(name)
        elif name + "Phase" in self._obj_map.keys():
            ret = self._obj_map.get(name + "Phase")
        else:
            raise KeyError(
                f"No object named '{name}' found in '{self._name}' registry!"
            )
        return ret

    def __contains__(self, name):
        return name in self._obj_map

    def __iter__(self):
        return iter(self._obj_map.items())

    def keys(self):
        return self._obj_map.keys()



from openai import OpenAI

from made.engine import ModelConfig
from made.engine.repository.base_engine_repository import BaseEngineRepository


class OllamaEngineRepositoryImpl(BaseEngineRepository):
    __instance = None

    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)

        return cls.__instance

    @classmethod
    def get_instance(cls):
        if cls.__instance is None:
            cls.__instance = cls()

        return cls.__instance

    def get_engine(self, model_config: ModelConfig):
        engine = OpenAI(base_url=model_config.base_url, api_key=model_config.api_key)

        return engine



from dataclasses import dataclass
from typing import Dict, Optional, Any

from made.messages.entity.base_message import BaseMessage


@dataclass
class BaseChatMessage(BaseMessage):
    role_name: str
    meta_dict: Optional[Dict[str, str]]
    role: str
    content: str = ""
    refusal: str = None

    def set_user_role(self: BaseMessage):
        return self.__class__(
            role_name=self.role_name,
            meta_dict=self.meta_dict,
            role="user",
            content=self.content,
        )



from abc import ABC, abstractmethod


class FileToolRepository(ABC):
    @staticmethod
    @abstractmethod
    def create_empty_file(file_path):
        pass

    @staticmethod
    @abstractmethod
    def read_file(file_path):
        pass

    @staticmethod
    @abstractmethod
    def write_file(file_path, content):
        pass

    @staticmethod
    @abstractmethod
    def update_file(file_path, content):
        pass

    @staticmethod
    @abstractmethod
    def delete_file(file_path):
        pass

    @staticmethod
    @abstractmethod
    def get_file_name_from_text(text: str, regex: str = r"(\w+\.\w+)"):
        pass

    @staticmethod
    @abstractmethod
    def get_contents_from_text(text, regex):
        pass

    @staticmethod
    @abstractmethod
    def build_directory_structure(tree_structure):
        pass

    @staticmethod
    @abstractmethod
    def convert_tree_structure_to_dict(tree_structure):
        pass



from made.chat_env.repository.chat_env_repository_impl import ChatEnvRepositoryImpl
from made.engine import ModelConfig
from made.phase import PhaseRegistry
from made.phase.entity.phase_chat_turn_limit import PhaseChatTurnLimit
from made.phase.entity.phase_engine_config import PhaseEngineConfig
from made.phase.entity.phase_prompts import PhasePrompt
from made.phase.repository.base_phase_repository_impl import BasePhaseRepositoryImpl
from made.role_playing.entity.role_prompts import RolePrompt
from made.role_playing.entity.role_type import RoleType


@PhaseRegistry.register()
class DemandAnalysisPhaseRepositoryImpl(BasePhaseRepositoryImpl):
    def __init__(
        self,
        model_config: ModelConfig,
        phase_prompt: str = PhasePrompt.demand_analysis,
        assistant_role_name: str = RoleType.CPO,
        assistant_role_prompt: str = RolePrompt.CPO,
        user_role_name: str = RoleType.CEO,
        user_role_prompt: str = RolePrompt.CEO,
        chat_turn_limit: int = PhaseChatTurnLimit.demand_analysis,
        **kwargs,
    ):
        phaes_engine_config = PhaseEngineConfig.demand_analysis
        super().__init__(
            model_config=model_config,
            phase_prompt=phase_prompt,
            assistant_role_name=assistant_role_name,
            assistant_role_prompt=assistant_role_prompt,
            user_role_name=user_role_name,
            user_role_prompt=user_role_prompt,
            chat_turn_limit=chat_turn_limit,
            temperature=phaes_engine_config.temperature,
            top_p=phaes_engine_config.top_p,
            **kwargs,
        )

    def update_phase_states(self, env: ChatEnvRepositoryImpl):
        pass

    def update_env_states(self, env: ChatEnvRepositoryImpl) -> ChatEnvRepositoryImpl:
        env.states.modality = self.seminar_conclusion
        return env



from dataclasses import dataclass


@dataclass
class PhasePrompt:
    demand_analysis: str = \
"""
As the {assistant_role}, to satisfy the new user's demand and the product should be realizable, \
first, we should discuss to decide which product modality do we want the product to be. \
Note that we MUST ONLY discuss the product modality and finish this discuss in 3 turns. \
Once we all have expressed our opinion(s) and agree with the results of the discussion unanimously, \
any of us must actively terminate the discussion by replying with only one line, \
which starts with a single symbol "<INFO>", followed by our final product modality, for example "<INFO> Application".
"""

    language_choose: str = \
"""
According to the new user's task and some creative brainstorm ideas listed below:
Task: {task}.
"Modality: {modality}.
Ideas: {ideas}.
We have decided to complete the task through a executable software implemented via a programming language. \
As the {assistant_role}, to satisfy the new user's demand and make the software realizable, you should propose a concrete programming language. \
If python can complete this task via Python, please answer Python; otherwise, answer another programming language (e.g., Java, C++, etc,). \
Note that we must "ONLY discuss the target programming language". \
Once we all have expressed our opinion(s) and agree with the results of the discussion unanimously, \
any of us must actively terminate the discussion and conclude the best programming language we have discussed without any other words or reasons, \
return only one line using the format: "<INFO> programming_language". \
For example: "<INFO> python".
"""

    coding: str = \
"""
According to the new user's task and our software designs listed below:  
Task: {task}. 
Task description: {description}. 
Modality: {modality}. 
Programming Language: {language} 
Ideas:{ideas} 
We have decided to complete the task through a executable software with multiple files implemented vi {language}. \
As the {assistant_role}, to satisfy the new user's demands, you should write one or multiple files \
and make sure that every detail of the architecture is, in the end, implemented as code. {gui} \
Think step by step and reason yourself to the right decisions to make sure we get it right. \
You will first lay out the names of the core classes, functions, methods that will be necessary, as w l as a quick comment on their purpose. \
Then you will output the content of each file including complete code. Each file must strictly follow  markdown code block format, \
where the following tokens must be replaced such that FILENAME is the lowercase file name including the file extension, \
LANGUAGE in the programming language, DOCSTRING is a string literal specified in source code \
that is used to document a specific segment of code, and CODE is the original code:
FILENAME 
```LANGUAGE 
''' 
DOCSTRING 
''' 
CODE 
``` 
You will start with the main file, then go to the ones that are imported by that file, and so on. \
Please note that the code should be fully functional. Ensure to implement all functions. No placeholders (such as 'pass' in Python). \
When discuss finish, start with symbol "<INFO>"  and list all the codes in format above. Final output should be like:
"""

    code_complete: str = \
"""
According to the new user's task and our software designs listed below: 
Task: {task}. 
Modality: {modality}.
Programming Language: {language} 
Codes: {codes} 
Unimplemented File: {unimplemented_file} 
In our software, each file must strictly follow a markdown code block format, \
where the following tokens must be replaced such that FILENAME is the lowercase file name including the file extension, \
LANGUAGE in the programming language, \
DOCSTRING is a string literal specified in source code that is used to document a specific segment of code, \
and CODE is the original code: 
FILENAME 
```LANGUAGE
''' DOCSTRING
''' CODE
``` 
As the {assistant_role}, to satisfy the complete function of our developed software, you have to implement all methods \
in the {unimplemented_file} file which contains a unimplemented class. Now, implement all methods of the {unimplemented_file} \
and all other codes needed, then output the fully implemented codes, strictly following the required format.
"""

    code_review_comment: str = \
"""
According to the new user's task and our software designs:
Task: {task}.
Modality: {modality}.
Programming Language: {language}
Ideas: {ideas}
Codes:
{codes}
As the {assistant_role}, to make the software directly operable without further coding, ChatDev have formulated the following regulations:
1) all referenced classes should be imported;
2) all methods should be implemented;
3) all methods need to have the necessary comments;
4) no potential bugs;
5) The entire project conforms to the tasks proposed by the user;
6) most importantly, do not only check the errors in the code, but also the logic of code. \
Make sure that user can interact with generated software without losing any feature in the requirement;
Now, you should check the above regulations one by one and review the codes in detail, \
propose one comment with the highest priority about the codes, and give me instructions on how to fix. \
Tell me your comment with the highest priority and corresponding suggestions on revision. \
If the codes are perfect and you have no comment on them, return only one line like "<INFO> Finished".
"""

    code_review_modification: str = \
"""
According to the new user's task, our designed product modality, languages and ideas, \
our developed first-edition source codes are listed below:
Task: {task}.
Modality: {modality}.
Programming Language: {language}
Ideas: {ideas}
Codes: 
{codes}
Comments on Codes:
{comments}
In the software, each file must strictly follow a markdown code block format, where the following tokens must be replaced \
such that FILENAME is the lowercase file name including the file extension, LANGUAGE in the programming language, \
DOCSTRING is a string literal specified in source code that is used to document a specific segment of code, and \
CODE is the original code. Format:
FILENAME
```LANGUAGE
'''
DOCSTRING
'''
CODE
```
As the {assistant_role}, to satisfy the new user's demand and make the software creative, executive and robust, \
you should modify corresponding codes according to the comments. \
Then, output the full and complete codes with all bugs fixed based on the comments. Return all codes strictly following the required format.
"""

    test_error_summary: str = \
"""
Our developed source codes and corresponding test reports are listed below:
Programming Language: {language}
Source Codes:
{codes}
Test Reports of Source Codes:
{test_reports}
According to my test reports, please locate and summarize the bugs that cause the problem.
"""

    test_modification: str = \
"""
Our developed source codes and corresponding test reports are listed below:
Programming Language: {language}
Source Codes:
{codes}
Test Reports of Source Codes:
{test_reports}
Error Summary of Test Reports:
{error_summary}
Note that each file must strictly follow a markdown code block format, where the following tokens must be replaced \
such that FILENAME is the lowercase file name including the file extension, \
LANGUAGE in the programming language, DOCSTRING is a string literal specified in source code that is used to document \
a specific segment of code, and CODE is the original code:
FILENAME
```LANGUAGE
'''
DOCSTRING
'''
CODE
```
As the {assistant_role}, to satisfy the new user's demand and make the software execute smoothly and robustly, \
you should modify the codes based on the error summary. Now, use the format exemplified above and modify the problematic codes based on the error summary. \
Output the codes that you fixed based on the test reported and corresponding explanations (strictly follow the format defined above, \
including FILENAME, LANGUAGE, DOCSTRING and CODE; incomplete TODO codes are strictly prohibited). If no bugs are reported, \
please return only one line like <INFO> Finished.
"""

    manual: str = \
"""
The new user's task, our developed codes and required dependencies are listed:
Task: {task}.
Modality: {modality}.
Programming Language: {language}
Ideas: {ideas}
Codes: 
{codes}
Requirements:
{requirements}
As the {assistant_role}, by using Markdown, you should write a README.md file which is a detailed user manual to use the software, \
including introducing main functions of the software, how to install environment dependencies and how to use/play it. For example:
README.md
```
# LangChain
Building applications with LLMs through composability
Looking for the JS/TS version? Check out LangChain.js.
**Production Support:** As you move your LangChains into production, we'd love to offer more comprehensive support.
Please fill out this form and we'll set up a dedicated support Slack channel.
## Quick Install
`pip install langchain`
or
`conda install langchain -c conda-forge`
## 🤔 What is this?
Large language models (LLMs) are emerging as a transformative technology, enabling developers to build applications that they previously could not. However, using these LLMs in isolation is often insufficient for creating a truly powerful app - the real power comes when you can combine them with other sources of computation or knowledge.
This library aims to assist in the development of those types of applications. Common examples of these applications include:
**❓ Question Answering over specific documents**
- Documentation
- End-to-end Example: Question Answering over Notion Database
**🤖 Agents**
- Documentation
- End-to-end Example: GPT+WolframAlpha
## 📖 Documentation
Please see [here](https://python.langchain.com) for full documentation on:
- Getting started (installation, setting up the environment, simple examples)
- How-To examples (demos, integrations, helper functions)
- Reference (full API docs)
- Resources (high-level explanation of core concepts)
```
"""



import os
import pickle
from typing import List, Tuple, Optional

from made.chat_chain.repository.chat_chain_repository import ChatChainRepository
from made.chat_env.repository.chat_env_repository_impl import ChatEnvRepositoryImpl
from made.phase.repository.base_phase_repository_impl import BasePhaseRepositoryImpl
from made.tools.file.repository.file_tool_repository_impl import FileToolRepositoryImpl
from made.tools.git.repository.git_tool_repository_impl import GitToolRepositoryImpl


class ChatChainRepositoryImpl(ChatChainRepository):
    def execute_step(
        self,
        env: ChatEnvRepositoryImpl,
        phase: BasePhaseRepositoryImpl,
    ):
        phase.execute(env)

    def execute_chain(
        self,
        env: ChatEnvRepositoryImpl,
        phases: List[BasePhaseRepositoryImpl],
        save_chain: bool = False,
        file_path: Optional[str] = None,
    ):
        for phase in phases:
            self.execute_step(env=env, phase=phase)
            if save_chain:
                ChatChainRepositoryImpl.save_chain(
                    file_path=file_path, global_env=env, current_phase=phase.__class__.__name__
                )

    def preprocessing(self, env: ChatEnvRepositoryImpl):
        workspace = os.path.join(env.config.directory)
        os.makedirs(workspace, exist_ok=True)
        FileToolRepositoryImpl.create_empty_file(os.path.join(workspace, "__init__.py"))
        log_path = os.path.join(workspace, "logs")
        os.makedirs(log_path, exist_ok=True)
        if env.config.git_management:
            GitToolRepositoryImpl.create_gitignore(workspace)
            FileToolRepositoryImpl.update_file(
                os.path.join(workspace, ".gitignore"), "logs/"
            )

    # TODO clean directory(pycache, etc.)
    def postprocessing(self):
        pass

    @staticmethod
    def save_chain(
        file_path: str, global_env: ChatEnvRepositoryImpl, current_phase: str
    ) -> None:
        current_phase = current_phase.replace("RepositoryImpl", "").replace("Phase", "")
        chain_dict = {
            "env": global_env,
            "phase": current_phase,
        }
        with open(file_path, "wb") as f:
            pickle.dump(chain_dict, f)
        print()
        print(f"\033[31mChain saved on {file_path}.\033[0m")

    @staticmethod
    def load_chain(file_path: str) -> Tuple[ChatEnvRepositoryImpl, str]:
        try:
            with open(file_path, "rb") as f:
                chain_dict = pickle.load(f)
        except Exception:
            raise TypeError(f"\033[31m`chain_path` is not provided.\033[0m")

        env = chain_dict["env"]
        phase = chain_dict["phase"]
        print()
        print(f"\033[31mChain Loaded from {file_path}.\nStarting from [{phase}] phase...\033[0m")
        return env, phase



from abc import ABC, abstractmethod


class DockerToolRepository(ABC):
    @staticmethod
    @abstractmethod
    def get_client():
        pass

    @staticmethod
    @abstractmethod
    def pull_image():
        pass

    @staticmethod
    @abstractmethod
    def start_container():
        pass

    @staticmethod
    @abstractmethod
    def stop_container():
        pass

    @staticmethod
    @abstractmethod
    def get_container():
        pass

    @staticmethod
    @abstractmethod
    def exec_command():
        pass



from dataclasses import dataclass


@dataclass
class EnvConfig:
    task_prompt: str
    directory: str
    background_prompt: str = (
        "SI-follow is a software company powered by LLM multi-agent"
    )
    clear_structure: bool = True
    incremental_develop: bool = False
    git_management: bool = False
    gui_design: bool = True



from dataclasses import dataclass
from typing import Dict, Optional

from made.messages.entity.base_message import BaseMessage


@dataclass
class BaseSystemMessage(BaseMessage):
    role_name: str
    meta_dict: Optional[Dict[str, str]] = None
    role: str = "system"
    content: str = ""



from dataclasses import dataclass
from typing import Dict, Optional

from made.messages.entity.chat_message.base_chat_message import BaseChatMessage


@dataclass
class AssistantChatMessage(BaseChatMessage):
    role_name: str
    meta_dict: Optional[Dict[str, str]] = None
    role: str = "user"
    content: str = ""



from dataclasses import dataclass
from typing import Any, Dict, Optional, Union


@dataclass
class BaseMessage:
    role_name: str
    meta_dict: Optional[Dict[str, str]]
    role: str
    content: str
    function_call: Any = None
    tool_calls: Any = None

    def __getattribute__(self, name: str) -> Any:
        delegate_methods = [method for method in dir(str) if not method.startswith("_")]
        if name in delegate_methods:
            content = super().__getattribute__("content")
            if isinstance(content, str):
                content_method = getattr(content, name, None)
                if callable(content_method):

                    def modify_arg(arg: Any) -> Any:
                        if isinstance(arg, BaseMessage):
                            return arg.content
                        elif isinstance(arg, (list, tuple)):
                            return type(arg)(modify_arg(item) for item in arg)
                        else:
                            return arg

                    def wrapper(*args: Any, **kwargs: Any) -> Any:
                        modified_args = [modify_arg(arg) for arg in args]
                        modified_kwargs = {k: modify_arg(v) for k, v in kwargs.items()}
                        output = content_method(*modified_args, **modified_kwargs)
                        return (
                            self._create_new_instance(output)
                            if isinstance(output, str)
                            else output
                        )

                    return wrapper

        return super().__getattribute__(name)

    def _create_new_instance(self, content: str) -> "BaseMessage":
        return self.__class__(
            role_name=self.role_name,
            meta_dict=self.meta_dict,
            role=self.role,
            content=content,
        )

    def __add__(self, other: Any) -> Union["BaseMessage", Any]:
        if isinstance(other, BaseMessage):
            combined_content = self.content.__add__(other.content)
        elif isinstance(other, str):
            combined_content = self.content.__add__(other)
        else:
            raise TypeError(
                f"Unsupported operand type(s) for +: '{type(self)}' and "
                f"'{type(other)}'"
            )
        return self._create_new_instance(combined_content)

    def __mul__(self, other: Any) -> Union["BaseMessage", Any]:
        if isinstance(other, int):
            multiplied_content = self.content.__mul__(other)
            return self._create_new_instance(multiplied_content)
        else:
            raise TypeError(
                f"Unsupported operand type(s) for *: '{type(self)}' and "
                f"'{type(other)}'"
            )

    def __len__(self) -> int:
        return len(self.content)

    def __contains__(self, item: str) -> bool:
        return item in self.content

    def to_message(self, role: Optional[str] = None) -> Dict[str, str]:
        role = role or self.role
        if role not in {"system", "user", "assistant"}:
            raise ValueError(f"Unrecognized role: {role}")
        return {"role": role, "content": self.content}

    def to_chat_message(
        self,
        role: Optional[str] = None,
    ) -> Dict[str, str]:
        role = role or self.role
        if role not in {"user", "assistant"}:
            raise ValueError(f"Unrecognized role: {role}")
        return {"role": role, "content": self.content}

    def to_system_message(self) -> Dict[str, str]:
        return {"role": "system", "content": self.content}

    def to_user_message(self) -> Dict[str, str]:
        return {"role": "user", "content": self.content}

    def to_assistant_message(self) -> Dict[str, str]:
        return {"role": "assistant", "content": self.content}



from dataclasses import dataclass
from typing import Dict, Optional

from made.messages.entity.system_message.base_system_message import BaseSystemMessage


@dataclass
class AssistantSystemMessage(BaseSystemMessage):
    role_name: str
    meta_dict: Optional[Dict[str, str]] = None
    role: str = "system"
    content: str = ""



from typing import Any, Dict, List, Optional

from tenacity import retry, stop_after_attempt, wait_exponential

from made.agent.entity.agent_response import ChatAgentResponse
from made.agent.repository.agent_repository_impl import AgentRepositoryImpl
from made.agent.service.agent_service import Agentservice
from made.engine import ModelConfig
from made.messages.entity.chat_message.base_chat_message import BaseChatMessage
from made.messages.entity.system_message.base_system_message import BaseSystemMessage


class AgentServiceImpl(Agentservice):
    def __init__(
        self,
        system_message: BaseSystemMessage,
        model_config: ModelConfig,
        message_window_size: int = None,
    ):
        self.agent_repository = AgentRepositoryImpl(
            system_message, model_config, message_window_size
        )
        self.terminated = False

    @retry(wait=wait_exponential(min=5, max=60), stop=stop_after_attempt(5))
    def step(
        self,
        input_message: BaseChatMessage,
    ) -> ChatAgentResponse:
        messages = self.agent_repository.update_messages(input_message)
        if (
            self.agent_repository.message_window_size is not None
            and len(messages) > self.agent_repository.message_window_size
        ):
            messages = [self.agent_repository.system_message] + messages[
                -self.agent_repository.message_window_size :
            ]
        messages = [message.to_message() for message in messages]

        try:
            response = self.agent_repository.engine.run(
                messages=messages, ollama_config=self.agent_repository.model_config
            )
            output_messages = [
                BaseChatMessage(
                    role_name=self.agent_repository.role_name,
                    meta_dict=dict(),
                    **dict(choice.message),
                )
                for choice in response.choices
            ]
            info = self.agent_repository.get_info(
                response.id,
                response.usage,
                [str(choice.finish_reason) for choice in response.choices],
                response.usage.total_tokens,
            )

            if output_messages[0].content.split("\n")[-1].startswith("<INFO>"):
                self.agent_repository.info = True

        except Exception as e:
            print(f"error occurred: {e}")
            self.terminated = True
            output_messages = []

            info = self.agent_repository.get_info(
                None,
                None,
                ["max_tokens_exceeded"],
                response.usage.total_tokens,
            )

        return ChatAgentResponse(output_messages, self.terminated, info)



from dataclasses import dataclass


@dataclass
class RolePrompt:
    CEO: str = """
{background_prompt}, You are Chief Executive Officer. \
Now, we are both working at SI-follow and we share a common interest in collaborating to successfully complete a task assigned by a new customer. \
Your main responsibilities include being an active decision-maker on users' demands and other key policy issues, leader, manager, and executor. \
Your decision-making role involves high-level decisions about policy and strategy; \
and your communicator role can involve speaking to the organization's management and employees. \
Here is a new customer's task: {task}. To complete the task, I will give you one or more instructions, \
and you must help me to write a specific solution that appropriately solves the requested instruction based on your expertise and my needs.
"""

    CTO: str = """
{background_prompt} You are Chief Technology Officer. we are both working at SI-follow. \
We share a common interest in collaborating to successfully complete a task assigned by a new customer. \
You are very familiar to information technology. You will make high-level decisions for the overarching technology infrastructure \
that closely align with the organization's goals, while you work alongside the organization's information technology (\"IT\") staff members \
to perform everyday operations. Here is a new customer's task: {task}. \
To complete the task, You must write a response that appropriately solves the requested instruction based on your expertise and customer's needs.
"""

    CPO: str = """
{background_prompt} You are Chief Product Officer. we are both working at SI-follow. \
We share a common interest in collaborating to successfully complete a task assigned by a new customer. \
You are responsible for all product-related matters in SI-follow. \
Usually includes product design, product strategy, product vision, product innovation, project management and product marketing. \
Here is a new customer's task: {task}. To complete the task, you must write a response that appropriately solves the requested instruction \
based on your expertise and customer's needs.
"""

    COUNSELOR: str = """
{background_prompt} You are Counselor. \
Now, we share a common interest in collaborating to successfully complete a task assigned by a new customer. \
Your main responsibilities include asking what user and customer think and provide your valuable suggestions. \
Here is a new customer's task: {task}. To complete the task, I will give you one or more instructions, \
and you must help me to write a specific solution that appropriately solves the requested instruction based on your expertise and my needs.
"""

    PROGRAMMER: str = """
{background_prompt} , You are Programmer. we are both working at SI-follow. \
We share a common interest in collaborating to successfully complete a task assigned by a new customer. \
You can write/create computer software or applications by providing a specific programming language to the computer. \
You have extensive computing and coding experience in many varieties of programming languages and platforms, such as Python, Java, C, C++, HTML, CSS, JavaScript, XML, SQL, PHP, etc,. \
Here is a new customer's task: {task}. To complete the task, you must write a response that appropriately solves the requested instruction \
based on your expertise and customer's needs.
"""

    REVIEWER: str = """
{background_prompt} You are Code Reviewer. we are both working at SI-follow. \
We share a common interest in collaborating to successfully complete a task assigned by a new customer. \
You can help programmers to assess source codes for software troubleshooting, fix bugs to increase code quality and robustness, \
and offer proposals to improve the source codes. Here is a new customer's task: {task}.  \
To complete the task, you must write a response that appropriately solves the requested instruction based on your expertise and customer's needs.
"""

    TESTER: str = """
{background_prompt} You are Software Test Engineer. we are both working at SI-follow. \
We share a common interest in collaborating to successfully complete a task assigned by a new customer. \
You can use the software as intended to analyze its functional properties, design manual and automated test procedures \
to evaluate each software product, build and implement software evaluation test programs, \
and run test programs to ensure that testing protocols evaluate the software correctly. Here is a new customer's task: {task}. \
To complete the task, you must write a response that appropriately solves the requested instruction based on your expertise and customer's needs.
"""



import logging
from typing import Optional


class Logger:
    def __init__(
        self,
        logger_name: str,
        log_file_path: Optional[str] = None,
        log_level=logging.INFO,
    ):
        self.__logger = logging.getLogger(logger_name)
        self.__logger.setLevel(log_level)

        self.__logger.handlers = []
        formatter = logging.Formatter(
            "[%(asctime)s] - [%(filename)s file line:%(lineno)d] - %(levelname)s: %(message)s"
        )

        if log_file_path is not None:
            file_handler = logging.FileHandler(log_file_path)
            file_handler.setFormatter(formatter)
            self.__logger.addHandler(file_handler)

        console_handler = logging.StreamHandler()
        console_handler.setFormatter(formatter)
        self.__logger.addHandler(console_handler)

    def get_logger(self) -> logging.Logger:
        return self.__logger



from copy import deepcopy
from typing import Dict, Union, Optional

from made.agent.service.agent_service_impl import AgentServiceImpl
from made.chat_env.repository.chat_env_repository_impl import ChatEnvRepositoryImpl
from made.messages.entity.chat_message.user_message import UserChatMessage
from made.engine import ModelConfig
from made.phase.entity.phase_states import PhaseStates
from made.phase.repository.base_phase_repository import BasePhaseRepository
from made.role_playing.service.role_playing_service_impl import RolePlayingServiceImpl


class BasePhaseRepositoryImpl(BasePhaseRepository):
    def __init__(
        self,
        model_config: ModelConfig,
        phase_prompt: str,
        assistant_role_name: str,
        assistant_role_prompt: str,
        user_role_name: str,
        user_role_prompt: str,
        chat_turn_limit: int = 10,
        **kwargs
    ):
        model_config = deepcopy(model_config)
        self.model_config = model_config
        if temperature := kwargs.get("temperature"):
            if temperature is not None:
                self.model_config.temperature = temperature
        if top_p := kwargs.get("top_p"):
            if top_p is not None:
                self.model_config.top_p = top_p
        self.phase_prompt = phase_prompt
        self.assistant_role_name = assistant_role_name
        self.assistant_role_prompt = assistant_role_prompt
        self.user_role_name = user_role_name
        self.user_role_prompt = user_role_prompt
        self.chat_turn_limit = chat_turn_limit

        self.seminar_conclusion = None
        
        states = kwargs.get("states")
        self.states: Optional[Union[PhaseStates, Dict]] = states if states else PhaseStates()

    def chatting(
        self,
        env: ChatEnvRepositoryImpl,
        task_prompt: str,
        phase_prompt: str,
        assistant_role_name: str,
        assistant_role_prompt: str,
        user_role_name: str,
        user_role_prompt: str,
        placeholders=None,
        chat_turn_limit=10,
    ) -> str:
        if placeholders is None:
            placeholders = {}
        if not isinstance(placeholders, dict):
            placeholders = placeholders.__dict__
        assert 1 <= chat_turn_limit <= 100

        role_play_session = RolePlayingServiceImpl(
            model_config=self.model_config,
            task_prompt=task_prompt,
            assistant_role_name=assistant_role_name,
            assistant_role_prompt=assistant_role_prompt,
            user_role_name=user_role_name,
            user_role_prompt=user_role_prompt,
            background_prompt=env.config.background_prompt,
        )

        _, input_user_message = role_play_session.role_playing_repository.init_chat(
            placeholders, phase_prompt
        )
        print()
        print(
            f"\033[93m{input_user_message.role_name}\033[0m",
            ": ",
            input_user_message.content,
        )
        seminar_conclusion = None

        for _ in range(chat_turn_limit):
            assistant_response, user_response = role_play_session.step(
                input_user_message, chat_turn_limit == 1
            )
            print()
            print(
                f"\033[93m{assistant_response.message.role_name}\033[0m",
                ": ",
                assistant_response.message.content,
            )
            if user_response.message is not None:
                print()
                print(
                    f"\033[93m{user_response.message.role_name}\033[0m",
                    ": ",
                    user_response.message.content,
                )
            if (
                role_play_session.role_playing_repository.assistant_agent.agent_repository.info
            ):
                seminar_conclusion = assistant_response.message.content
                break
            if assistant_response.terminated:
                break

            if (
                role_play_session.role_playing_repository.user_agent.agent_repository.info
            ):
                seminar_conclusion = user_response.message.content
                break
            if user_response.terminated:
                break

            if chat_turn_limit > 1:
                input_user_message = user_response.message
            else:
                break
        
        # TODO seminar conclusion should be more clear
        if seminar_conclusion is None:
            conversations = (
                role_play_session.role_playing_repository.assistant_agent.agent_repository.stored_messages
            )
            todo = conversations[0].content
            conversations = "".join(
                [conversation.content for conversation in conversations[1:]]
            )
            conclude_prompt = (
                f"Below are conversation between {user_role_name} and {assistant_role_name}.:\nConversations: {conversations}"
                + f'\nYou have to conclude conversations and answer for "{todo}".\n If conversation contains code, don\'t modify the code.'
            )
            message = UserChatMessage("User", content=conclude_prompt)
            agent = AgentServiceImpl(
                role_play_session.role_playing_repository.user_system_messsge,
                self.model_config,
            )
            seminar_conclusion = agent.step(message).message.content
            if "<INFO>" not in seminar_conclusion:
                seminar_conclusion = "<INFO>" + seminar_conclusion

        seminar_conclusion = seminar_conclusion.split("<INFO>")[-1]
        print()
        print(f"\033[31mseminar conclusion\033[0m: {seminar_conclusion}")
        return seminar_conclusion

    def update_phase_states(self, env: ChatEnvRepositoryImpl):
        raise NotImplementedError

    def update_env_states(self, env: ChatEnvRepositoryImpl):
        raise NotImplementedError

    def execute(
        self,
        env: ChatEnvRepositoryImpl,
    ) -> ChatEnvRepositoryImpl:
        self.update_phase_states(env)
        self.seminar_conclusion = self.chatting(
            env=env,
            task_prompt=env.config.task_prompt,
            phase_prompt=self.phase_prompt,
            assistant_role_name=self.assistant_role_name,
            assistant_role_prompt=self.assistant_role_prompt,
            user_role_name=self.user_role_name,
            user_role_prompt=self.user_role_prompt,
            placeholders=self.states,
            chat_turn_limit=self.chat_turn_limit,
        )
        env = self.update_env_states(env)
        return env



from dataclasses import dataclass
from typing import Any, Dict, List

from made.messages.entity.chat_message.base_chat_message import BaseChatMessage


@dataclass(frozen=True)
class ChatAgentResponse:
    messages: List[BaseChatMessage]
    terminated: bool
    info: Dict[str, Any]

    @property
    def message(self):
        if self.terminated:
            raise RuntimeError("error in AgentResponse, info:{}".format(str(self.info)))
        if len(self.messages) > 1:
            raise RuntimeError(
                "Property message is only available for a single message in messages"
            )
        elif len(self.messages) == 0:
            if len(self.info) > 0:
                raise RuntimeError(
                    "Empty messages in AgentResponse, info:{}".format(str(self.info))
                )
            else:
                return None
        return self.messages[0]



from abc import ABC, abstractmethod


class AgentRepository(ABC):
    @abstractmethod
    def reset(self):
        pass

    @abstractmethod
    def get_info(self, id, usage, termination_reasons, num_tokens):
        pass

    @abstractmethod
    def init_messages(self):
        pass

    @abstractmethod
    def update_messages(self, message):
        pass



from openai.types.chat.chat_completion import ChatCompletion

from made.engine import ModelConfig
from made.engine.repository.ollama.ollama_engine_repository_impl import (
    OllamaEngineRepositoryImpl,
)
from made.engine.service.base_engine_service import BaseEngineService


class OllamaEngineServiceImpl(BaseEngineService):
    __instance = None

    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
            cls.__instance.__ollama_engine_repository = (
                OllamaEngineRepositoryImpl.get_instance()
            )

        return cls.__instance

    @classmethod
    def get_instance(cls):
        if cls.__instance is None:
            cls.__instance = cls()

        return cls.__instance

    def run(self, messages, ollama_config: ModelConfig) -> ChatCompletion:
        engine = self.__ollama_engine_repository.get_engine(ollama_config)

        response = engine.chat.completions.create(
            model=ollama_config.model,
            messages=messages,
            max_tokens=ollama_config.max_tokens,
            temperature=ollama_config.temperature,
            top_p=ollama_config.top_p,
            stream=ollama_config.stream,
            frequency_penalty=ollama_config.frequency_penalty,
            presence_penalty=ollama_config.presence_penalty,
        )

        return response



import os
import re
from typing import Dict

from made.tools.file.repository.file_tool_repository import FileToolRepository


class FileToolRepositoryImpl(FileToolRepository):
    @staticmethod
    def create_empty_file(file_path: str):
        with open(file_path, "w") as f:
            pass

    @staticmethod
    def read_file(file_path: str) -> str:
        with open(file_path, "r") as f:
            content = f.read()
        return content

    @staticmethod
    def write_file(file_path: str, content: str) -> None:
        with open(file_path, "w") as f:
            f.write(content)

    @staticmethod
    def update_file(file_path: str, content: str) -> None:
        with open(file_path, "a") as f:
            f.write(content)

    @staticmethod
    def delete_file(file_path) -> None:
        os.remove(file_path)

    @staticmethod
    def get_file_name_from_text(text: str, regex: str = r"(\w+\.\w+)"):
        for candidate in re.finditer(regex, text, re.DOTALL):
            file_name = candidate.group()
            file_name = file_name.lower()
        return file_name

    @staticmethod
    def abstract_contents_from_text(
        text: str, regex: str = r"\n(.+?)\n```.*?\n(.*?)```"
    ) -> Dict[str, str]:
        contents = {}
        matches = re.finditer(regex, text, re.DOTALL)
        for match in matches:
            file_name, content = match.groups()
            file_name = FileToolRepositoryImpl.get_file_name_from_text(file_name)
            contents[file_name] = content

        return contents

    @staticmethod
    def build_directory_structure(structure: Dict[str, str]):
        pass

    @staticmethod
    def convert_tree_structure_to_dict(tree_structure: str) -> Dict[str, str]:
        pass



from typing import Any, Dict, List, Optional, Union

from made.agent.repository.agent_repository import AgentRepository
from made.engine import ModelConfig
from made.engine.service.ollama.ollama_engine_service_impl import OllamaEngineServiceImpl
from made.messages import MessageType
from made.messages.entity.chat_message.base_chat_message import BaseChatMessage
from made.messages.entity.system_message.base_system_message import BaseSystemMessage


class AgentRepositoryImpl(AgentRepository):
    def __init__(
        self,
        system_message: BaseSystemMessage,
        model_config: ModelConfig,
        message_window_size: Optional[int] = None,
    ) -> None:
        self.system_message: BaseSystemMessage = system_message
        self.role_name: str = system_message.role_name
        self.engine: Union[OllamaEngineServiceImpl] = (
            OllamaEngineServiceImpl.get_instance()
        )
        self.model_config: ModelConfig = model_config
        self.model_token_limit: int = model_config.max_tokens
        self.message_window_size: Optional[int] = message_window_size
        self.terminated: bool = False
        self.info: bool = False
        self.init_messages()

    def reset(self) -> List[MessageType]:
        self.terminated = False
        self.init_messages()
        return self.stored_messages

    def get_info(
        self,
        id: Optional[str],
        usage: Optional[Dict[str, int]],
        termination_reasons: List[str],
        num_tokens: int,
    ) -> Dict[str, Any]:
        return {
            "id": id,
            "usage": usage,
            "termination_reasons": termination_reasons,
            "num_tokens": num_tokens,
        }

    def init_messages(self) -> None:
        self.stored_messages: List[MessageType] = [self.system_message]

    def update_messages(self, message: BaseChatMessage) -> List[MessageType]:
        self.stored_messages.append(message)
        return self.stored_messages



import copy
from typing import Dict, List, Optional, Sequence

from made.agent.service.agent_service_impl import AgentServiceImpl
from made.engine import ModelConfig
from made.messages.entity.chat_message.base_chat_message import BaseChatMessage
from made.messages.entity.chat_message.user_message import UserChatMessage
from made.messages.entity.system_message.assistant_system_message import (
    AssistantSystemMessage,
)
from made.messages.entity.system_message.user_system_message import UserSystemMessage
from made.role_playing.repository.role_playing_repository import RolePlayingRepository


class RolePlayingRepositoryImpl(RolePlayingRepository):
    def __init__(
        self,
        model_config: ModelConfig,
        task_prompt: str,
        assistant_role_name: str,
        assistant_role_prompt: str,
        user_role_name: str,
        user_role_prompt: str,
        assistant_agent_kwargs: Optional[Dict] = None,
        user_agent_kwargs: Optional[Dict] = None,
        extend_system_message_meta_dicts: Optional[List[Dict]] = None,
        background_prompt: Optional[str] = "",
    ) -> None:
        system_message_meta_dicts = [
            dict(background_prompt=background_prompt, task=task_prompt)
        ] * 2
        extend_system_message_meta_dicts = [
            dict(assistant_role=assistant_role_name, user_role=user_role_name)
        ] * 2
        if extend_system_message_meta_dicts is not None:
            system_message_meta_dicts = [
                {**system_message_meta_dict, **extend_system_message_meta_dict}
                for system_message_meta_dict, extend_system_message_meta_dict in zip(
                    system_message_meta_dicts, extend_system_message_meta_dicts
                )
            ]

        self.assistant_system_message = AssistantSystemMessage(
            role_name=assistant_role_name,
            meta_dict=system_message_meta_dicts[0],
            content=assistant_role_prompt.format(**system_message_meta_dicts[0]),
        )
        self.user_system_messsge = UserSystemMessage(
            role_name=user_role_name,
            meta_dict=system_message_meta_dicts[1],
            content=user_role_prompt.format(**system_message_meta_dicts[1]),
        )

        self.assistant_agent = AgentServiceImpl(
            system_message=self.assistant_system_message,
            model_config=model_config,
            **(assistant_agent_kwargs or {}),
        )
        self.user_agent = AgentServiceImpl(
            system_message=self.user_system_messsge,
            model_config=model_config,
            **(user_agent_kwargs or {}),
        )

    def init_chat(self, placeholders={}, phase_prompt=None):
        self.assistant_agent.agent_repository.reset()
        self.user_agent.agent_repository.reset()

        content = phase_prompt.format(
            **(
                {"assistant_role": self.assistant_agent.agent_repository.role_name}
                | placeholders
            )
        )
        user_message = UserChatMessage(
            role_name=self.user_system_messsge.role_name, role="user", content=content
        )
        pseudo_message = copy.deepcopy(user_message)
        pseudo_message.role = "assistant"
        self.user_agent.agent_repository.update_messages(pseudo_message)

        return None, user_message

    def process_messages(
        self,
        messages: Sequence[BaseChatMessage],
    ) -> BaseChatMessage:
        if len(messages) == 0:
            raise ValueError("No messages to process.")
        if len(messages) > 1:
            raise ValueError(
                "Got more than one message to process. "
                f"Num of messages: {len(messages)}."
            )
        else:
            processed_message = messages[0]

        return processed_message



from typing import Dict, List, Optional, Tuple

from made.agent.entity.agent_response import ChatAgentResponse
from made.engine import ModelConfig
from made.messages.entity.chat_message.base_chat_message import BaseChatMessage
from made.role_playing.repository.role_playing_repository_impl import (
    RolePlayingRepositoryImpl,
)
from made.role_playing.service.role_playing_service import RolePlayingService


class RolePlayingServiceImpl(RolePlayingService):
    def __init__(
        self,
        model_config: ModelConfig,
        task_prompt: str,
        assistant_role_name: str,
        assistant_role_prompt: str,
        user_role_name: str,
        user_role_prompt: str,
        assistant_agent_kwargs: Optional[Dict] = None,
        user_agent_kwargs: Optional[Dict] = None,
        extend_system_message_meta_dicts: Optional[List[Dict]] = None,
        background_prompt: Optional[str] = "",
    ):
        self.role_playing_repository = RolePlayingRepositoryImpl(
            model_config=model_config,
            task_prompt=task_prompt,
            assistant_role_name=assistant_role_name,
            assistant_role_prompt=assistant_role_prompt,
            user_role_name=user_role_name,
            user_role_prompt=user_role_prompt,
            assistant_agent_kwargs=assistant_agent_kwargs,
            user_agent_kwargs=user_agent_kwargs,
            extend_system_message_meta_dicts=extend_system_message_meta_dicts,
            background_prompt=background_prompt,
        )

    def step(
        self,
        user_message: BaseChatMessage,
        assistant_only: bool,
    ) -> Tuple[ChatAgentResponse, ChatAgentResponse]:
        assert isinstance(user_message, BaseChatMessage), print(
            "broken user_message: " + str(user_message)
        )

        user_message_set_role = user_message.set_user_role()
        assistant_response = self.role_playing_repository.assistant_agent.step(
            user_message_set_role
        )
        if assistant_response.terminated or assistant_response.messages is None:
            return (
                ChatAgentResponse(
                    [assistant_response.messages],
                    assistant_response.terminated,
                    assistant_response.info,
                ),
                ChatAgentResponse([], False, {}),
            )
        assistant_message = self.role_playing_repository.process_messages(
            assistant_response.messages
        )
        if self.role_playing_repository.assistant_agent.agent_repository.info:
            return (
                ChatAgentResponse(
                    [assistant_message],
                    assistant_response.terminated,
                    assistant_response.info,
                ),
                ChatAgentResponse([], False, {}),
            )
        self.role_playing_repository.assistant_agent.agent_repository.update_messages(
            assistant_message
        )

        if assistant_only:
            return (
                ChatAgentResponse(
                    [assistant_message],
                    assistant_response.terminated,
                    assistant_response.info,
                ),
                ChatAgentResponse([], False, {}),
            )

        assistant_message_set_role = assistant_message.set_user_role()
        user_response = self.role_playing_repository.user_agent.step(
            assistant_message_set_role
        )
        if user_response.terminated or user_response.messages is None:
            return (
                ChatAgentResponse(
                    [assistant_message],
                    assistant_response.terminated,
                    assistant_response.info,
                ),
                ChatAgentResponse(
                    [user_response], user_response.terminated, user_response.info
                ),
            )
        user_message = self.role_playing_repository.process_messages(
            user_response.messages
        )
        if self.role_playing_repository.user_agent.agent_repository.info:
            return (
                ChatAgentResponse(
                    [assistant_message],
                    assistant_response.terminated,
                    assistant_response.info,
                ),
                ChatAgentResponse(
                    [user_message], user_response.terminated, user_response.info
                ),
            )
        self.role_playing_repository.user_agent.agent_repository.update_messages(
            user_message
        )

        return (
            ChatAgentResponse(
                [assistant_message],
                assistant_response.terminated,
                assistant_response.info,
            ),
            ChatAgentResponse(
                [user_message], user_response.terminated, user_response.info
            ),
        )



from made.engine import ModelConfig
from made.phase import PhaseRegistry
from made.phase.repository.base_phase_repository_impl import BasePhaseRepositoryImpl
from made.phase.service.phase_service import PhaseService


class PhaseServiceImpl(PhaseService):
    def __init__(
        self,
        model_config: ModelConfig,
    ):
        self.model_config = model_config

    def get_phase(
        self,
        phase_name: str,
        **kwargs,
    ) -> BasePhaseRepositoryImpl:
        return PhaseRegistry.get(phase_name)(
            model_config=self.model_config,
            **kwargs,
        )



from abc import ABC, abstractmethod


class BasePhaseRepository(ABC):
    @abstractmethod
    def chatting(
        self,
        env,
        task_prompt,
        phase_prompt,
        assistant_role_name,
        assistant_role_prompt,
        user_role_name,
        user_role_prompt,
        placeholders,
        chat_turn_limit,
    ):
        pass

    @abstractmethod
    def update_phase_states(self, env_states):
        pass

    @abstractmethod
    def update_env_states(self, env_states):
        pass



import os
import time
from typing import Optional

import docker
from docker import DockerClient
from docker.models.containers import Container

from made.tools.docker.repository.docker_tool_repository import DockerToolRepository


class DockerToolRepositoryImpl(DockerToolRepository):
    @staticmethod
    def get_client() -> DockerClient:
        return docker.from_env()

    @staticmethod
    def pull_image(client: DockerClient, image_name: str = "python:3.9-slim"):
        repository, tag = image_name.split(":")
        client.images.pull(repository, tag)

    @staticmethod
    def start_container(container: Container):
        container.start()

    @staticmethod
    def stop_container(container: Container):
        container.stop()

    @staticmethod
    def get_container(
        container_name: str,
        image_name: Optional[str] = "python:3.9-slim",
        port: Optional[int] = 7979,
        volume: Optional[str] = "project_zoo",
        wait_delay: Optional[int] = 3,
    ) -> Container:
        client = DockerToolRepositoryImpl.get_client()
        containers = [container.name for container in client.containers.list(all=True)]
        if container_name in containers:
            container = client.containers.get(container_name)
            if container.status != "running":
                DockerToolRepositoryImpl.start_container(container)
        else:
            images = [image.attrs["RepoTags"] for image in client.images.list(all=True)]
            if image_name not in images:
                DockerToolRepositoryImpl.pull_image(client, image_name)

            image = client.images.get(image_name)
            container = client.containers.run(
                image=image,
                name=container_name,
                detach=True,
                tty=True,
                ports={f"{port}/tcp": port},
                volumes={os.path.abspath(volume): {"bind": f"/{volume}", "mode": "rw"}},
            )
        time.sleep(wait_delay)

        if not container:
            raise RuntimeError(f"Failed to create and start `{container_name}`")

        return container

    @staticmethod
    def exec_command(container: Container, command: str):
        res = container.exec_run(command)
        exit_code = res.exit_code
        output = res.output.decode()

        return output, exit_code



from abc import ABC, abstractmethod


class RolePlayingRepository(ABC):
    @abstractmethod
    def init_chat(self, placeholders, phase_prompt):
        pass

    @abstractmethod
    def process_messages(self, messages):
        pass



from dataclasses import dataclass


@dataclass
class PhaseChatTurnLimit:
    demand_analysis: int = 2
    language_choose: int = 2
    coding: int = 5
    code_complete: int = 3
    code_review_comment: int = 1
    code_review_modification: int = 1
    test_error_summary: int = 1
    test_modification: int = 1
    manual: int = 4



import importlib
import pkgutil

from made.utils.registry import Registry


PhaseRegistry = Registry("Phase")


def import_all_modules(package_name):
    package = importlib.import_module(package_name)
    for _, module_name, is_pkg in pkgutil.iter_modules(package.__path__):
        full_module_name = f"{package_name}.{module_name}"
        importlib.import_module(full_module_name)
        if is_pkg:
            import_all_modules(full_module_name)


import_all_modules("made.phase.repository")


from dataclasses import dataclass, field
from typing import Dict, List


@dataclass
class EnvStates:
    task_description: str = ""
    modality: str = ""
    ideas: str = ""
    language: str = ""
    review_comments: List[str] = field(default_factory=list)
    error_summary: List[str] = field(default_factory=list)
    test_reports: List[str] = field(default_factory=list)
    codes: Dict[str, str] = field(default_factory=dict)
    manual: Dict[str, str] = field(default_factory=dict)
    requirements: Dict[str, str] = field(default_factory=dict)



from dataclasses import dataclass


@dataclass
class EngineParams:
    temperature: float = 1.0
    top_p: float = 1.0


@dataclass
class PhaseEngineConfig:
    demand_analysis: EngineParams = EngineParams(temperature=0.7, top_p=0.8)
    language_choose: EngineParams = EngineParams(temperature=0.3, top_p=0.1)
    coding: EngineParams = EngineParams(temperature=0.2, top_p=0.1)
    code_complete: EngineParams = EngineParams(temperature=0.1, top_p=0.1)
    code_review_comment: EngineParams = EngineParams(temperature=0.1, top_p=0.1)
    code_review_modification: EngineParams = EngineParams(temperature=0.2, top_p=0.1)
    test_error_summary: EngineParams = EngineParams(temperature=0.1, top_p=0.1)
    test_modification: EngineParams = EngineParams(temperature=0.1, top_p=0.1)
    manual: EngineParams = EngineParams(temperature=0.7, top_p=0.4)



from abc import ABC, abstractmethod


class GitToolRepository(ABC):
    @staticmethod
    @abstractmethod
    def git_init():
        pass

    @staticmethod
    @abstractmethod
    def create_gitignore():
        pass

    @staticmethod
    @abstractmethod
    def git_commit():
        pass

    @staticmethod
    @abstractmethod
    def git_add():
        pass



import os
from typing import List, Optional

from made.chat_chain.repository.chat_chain_repository_impl import (
    ChatChainRepositoryImpl,
)
from made.chat_chain.service.chat_chain_service import ChatChainService
from made.chat_env.entity.env_config import EnvConfig
from made.chat_env.entity.env_states import EnvStates
from made.chat_env.repository.chat_env_repository_impl import ChatEnvRepositoryImpl
from made.engine.entity.ollama_config import OllamaConfig
from made.phase.service.phase_service_impl import PhaseServiceImpl


class ChatChainServiceImpl(ChatChainService):
    def __init__(
        self,
        task_prompt: str,
        directory: str,
        phases: List[str] = [
            "DemandAnalysis",
            "LanguageChoose",
            "Coding",
            "CodeComplete",
            "CodeReviewComment",
            "CodeReviewModification",
            "TestErrorSummary",
            "TestModification",
            "Manual",
        ],
        base_url: str = "http://127.0.0.1:11434/v1/",
        model: str = "llama3.2",
        api_key: str = "ollama",
        max_tokens: int = 40000,
        git_management: bool = False,
        save_chain: bool = False,
        save_name: Optional[str] = "chain",
        env_states: EnvStates = EnvStates(),
        **kwargs,
    ):
        self.model_config = OllamaConfig(
            base_url=base_url, model=model, api_key=api_key, max_tokens=max_tokens
        )
        self.chat_chain_repository = ChatChainRepositoryImpl()
        self.phase_service = PhaseServiceImpl(
            model_config=self.model_config,
        )

        env_config = EnvConfig(
            task_prompt=task_prompt, directory=directory, git_management=git_management
        )

        self.env = ChatEnvRepositoryImpl(env_config=env_config, env_states=env_states)
        self.save_chain = save_chain
        self.chain_path = os.path.join(directory, f"{save_name}.pkl")
        self.load_chain = os.path.exists(self.chain_path)
        if self.load_chain:
            env, last_phase = ChatChainRepositoryImpl.load_chain(self.chain_path)
            self.env = env
            self.phases = self.get_phases(phases[phases.index(last_phase) + 1 :])
        else:
            self.phases = self.get_phases(phases)

    def get_phases(self, phases: List[str]):
        phases = [self.phase_service.get_phase(phase) for phase in phases]
        return phases

    def run(self):
        if not self.load_chain:
            self.chat_chain_repository.preprocessing(self.env)
        self.chat_chain_repository.execute_chain(
            env=self.env,
            phases=self.phases,
            save_chain=self.save_chain,
            file_path=self.chain_path,
        )
        self.chat_chain_repository.postprocessing()



from abc import ABC, abstractmethod


class ChatChainRepository(ABC):
    @abstractmethod
    def execute_step(self, env, phase):
        pass

    @abstractmethod
    def execute_chain(self, env, phases):
        pass

    @abstractmethod
    def preprocessing(self):
        pass

    @abstractmethod
    def postprocessing(self):
        pass

    @abstractmethod
    def save_chain(self):
        pass

    @abstractmethod
    def load_chain(self):
        pass



from setuptools import setup, find_packages


setup(
    name="llm_made",
    version="0.0.6",
    description="multi-agent development",
    author="onebottlekick",
    author_email="gksqudcks97@gmail.com",
    url="https://github.com/onebottlekick/MADE",
    license="Apache-2.0",
    packages=find_packages(include=["made", "made.*"]),
    include_package_data=True,
    install_requires=["openai==1.45.1", "tenacity==9.0.0"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.9",
)



from dataclasses import dataclass
from typing import List, Optional, Union


@dataclass
class OllamaConfig:
    base_url: str = "http://127.0.0.1:11434/v1/"
    model: str = "llama3.2"
    api_key: str = "ollama"
    max_tokens: int = 40000
    stream: bool = False
    temperature: float = 1.0
    top_p: float = 1.0
    frequency_penalty: float = 0.0
    presence_penalty: float = 0.0
    stop: Optional[Union[str, List[str]]] = None
    seed: Optional[int] = None



from dataclasses import dataclass
from typing import List


@dataclass
class PhaseStates:
    task: str = ""
    description: str = ""
    modality: str = ""
    language: str = ""
    ideas: str = ""
    codes: str = ""
    comments: str = ""
    review_comments: str = ""
    test_reports: str = ""
    exist_bugs_flag: bool = False
    error_summary: str = ""
    cycle_index: int = 1
    unimplemented_file: str = ""
    max_num_implement: int = 5
    num_tried: int = 0
    modification_conclusion: str = ""
    gui: bool = False
    code_files: List[str] = ""
    requirements: str = ""



import os

from made.tools.git.repository.git_tool_repository import GitToolRepository
from made.tools.file.repository.file_tool_repository_impl import FileToolRepositoryImpl


class GitToolRepositoryImpl(GitToolRepository):
    @staticmethod
    def git_init(workspace_path: str):
        os.system(f"cd {workspace_path}; git init")

    @staticmethod
    def create_gitignore(workspace_path: str) -> None:
        FileToolRepositoryImpl.create_empty_file(
            os.path.join(workspace_path, ".gitignore")
        )

    @staticmethod
    def git_commit(workspace_path: str, message: str):
        os.system(f"cd {workspace_path}; git commit -m {message}")

    @staticmethod
    def git_add(workspace_path: str):
        os.system(f"cd {workspace_path}; git add .")



from made.chat_env.entity.env_config import EnvConfig
from made.chat_env.entity.env_states import EnvStates
from made.chat_env.repository.chat_env_repository import ChatEnvRepository


class ChatEnvRepositoryImpl(ChatEnvRepository):
    def __init__(self, env_config: EnvConfig, env_states: EnvStates = EnvStates()):
        self.reset(env_config, env_states)

    def reset(self, env_config: EnvConfig, env_states: EnvStates = EnvStates()):
        self.config = env_config
        self.states = env_states



from dataclasses import dataclass


@dataclass
class RoleType:
    CEO: str = "Chief Executive Officer"
    CPO: str = "Chief Product Officer"
    CTO: str = "Chief Technology Officer"
    COUNSELOR: str = "Counselor"
    PROGRAMMER: str = "Programmer"
    REVIEWER: str = "Code Reviewer"
    TESTER: str = "Software Test Engineer"
